/* FILE NAME: server.c */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "inv.h"                    /* header created by the IDL compiler   */
#include "status.h"                 /* contains the CHECK_STATUS macro      */
#define STRINGLEN 50

main (argc, argv)
int argc;
char *argv[];
{
   error_status_t       status;          /* error status                    */
                                         /* RPC vectors                     */
   rpc_binding_vector_t *binding_vector; /* binding handle list             */
   RPC_PROTSEQ_VECTOR   *protseq_vector; /*protocol sequence list           */

   char entry_name[STRINGLEN];           /* name service entry name         */
   char group_name[STRINGLEN];           /* name service group name         */
   char annotation[STRINGLEN];           /* annotation for endpoint map     */
   char hostname[STRINGLEN];             /* used to store the computer name */
   DWORD hostname_size=STRINGLEN;        /* required by GetComputerName     */
   /************************** REGISTER INTERFACE ***************************/
   status =
   RpcServerRegisterIf(
      inv_v1_0_s_ifspec,                 /* interface specification (inv.h) */
      NULL,
      NULL
   );
   CHECK_STATUS(status, "Can't register interface:", ABORT);

   /****************** CREATING SERVER BINDING INFORMATION ******************/
   if(argc > 1) {
      status =
      RpcServerUseProtseq(                 /* use a protocol sequence       */
         (unsigned char *)argv[1],         /* the input protocol sequence   */
         RPC_C_PROTSEQ_MAX_REQS_DEFAULT,   /* the default number of requests*/
         NULL                              /* security descriptor (not reqd)*/
      );
      CHECK_STATUS(status, "Can't use this protocol sequence:", ABORT);
   }
   else {
      puts("You can invoke the server with a protocol sequence argument.");
      status =
      RpcServerUseAllProtseqs(            /* use all protocol sequences     */
         RPC_C_PROTSEQ_MAX_REQS_DEFAULT,  /* the default number of requests */
         NULL                             /* securlty descriptor (not reqd) */
      );
      CHECK_STATUS(status, "Can't register protocol sequences:", ABORT);
   }

   status =
   RpcServerInqBindings(            /* get binding information for server   */
      &binding_vector
   );
   CHECK_STATUS(status, "Can't get binding information:", ABORT);

   /*************************** ADVERTISE SERVER ****************************/

   strcpy(entry_name, "/.:/inventory_");
   GetComputerName(&hostname, &hostname_size);
   strcat(entry_name, hostname);
   status =
   RpcNsBindingExport(             /* export to a name service database     */
      RPC_C_NS_SYNTAX_DEFAULT,     /* syntax of entry name                  */
      (unsigned char *)entry_name, /* name of entry in name service         */
      inv_v1_0_s_ifspec,           /* interface specification (inv.h)       */
      binding_vector,              /* binding information                   */
      NULL                         /* no object UUIDs exported              */
   );
   CHECK_STATUS(status, "Can't export to name service database:", RESUME);

   /*************************** MANAGE ENDPOINTS ****************************/
   strcpy(annotation, "Inventory interface");
   status =
   RpcEpRegister(                  /* add endpoints to local endpoint map   */
      inv_v1_0_s_ifspec,           /* interface specification (inv.h)       */
      binding_vector,              /* vector of server binding handles      */
      NULL,                        /* no object UUIDs to register           */
      (unsigned char *)annotation  /* annotation supplied (not required)    */
   );
   CHECK_STATUS(status, "Can't add endpoints to local endpoint map:", RESUME);

   status =
   RpcBindingVectorFree(                   /* free server binding handles   */
      &binding_vector
   );
   CHECK_STATUS(status, "Can't free server binding handles:", RESUME);

   open_inventory();                      /* application specific procedure */

   /******************* LISTEN FOR REMOTE PROCEDURE CALLS *******************/
   RpcTryExcept                        /* thread exception handling macro   */
   {
      status =
      RpcServerListen(
         1,                  /* process one remote procedure call at a time */
         RPC_C_LISTEN_MAX_CALLS_DEFAULT,
         NULL
      );
      CHECK_STATUS(status, "rpc listen failed:", RESUME);
   }
   RpcExcept (RpcExceptionCode())             /* error recovery and cleanup */
   {
      close_inventory();                  /* application specific procedure */
      status =
      RpcServerInqBindings(                    /* get binding information   */
         &binding_vector
      );
      CHECK_STATUS(status, "Can't get binding information:", RESUME);

      status =
      RpcEpUnregister(         /* remove endpoints from local endpoint map  */
         inv_v1_0_s_ifspec,    /* interface specification (inv.h)           */
         binding_vector,       /* vector of server binding handles          */
         NULL                  /* no object UUIDs                           */
      );
      CHECK_STATUS(status, "Can't remove endpoints from endpoint map:", RESUME);

      status =
      RpcBindingVectorFree(                 /* free server binding handles  */
         &binding_vector
      );
      CHECK_STATUS(status, "Can't free server binding handles:", RESUME);

      puts("\nServer quit!");
   }
   RpcEndExcept;
}  /* END SERVER INITIALIZATION */

/*************************************************************************/
/***                MIDL_user_allocate / MIDL_user_free                ***/
/*************************************************************************/

void * __RPC_API
MIDL_user_allocate
           (
            size
           )
size_t size;
{

    unsigned char * ptr;
    ptr = malloc( size );
    return ( (void *)ptr );

}

void __RPC_API
MIDL_user_free
           (
            object
           )
void * object;
{
    free (object);
}
